from .color_scheme import ColorScheme as ColorScheme
from .hsba_color import HSBAColor as HSBAColor

def print_color(color: HSBAColor, text: str = '  '): ...
def print_scheme(name: str, scheme: ColorScheme): ...
def main() -> None: ...
